<?php
/**
 * Philia Cashier Fields
 * The class responsible for managing the Philia Cashier fields in WordPress.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/10/13 17:49:58
 * License: MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

class Philia_Cashier_Fields {
    
    /**
     * Holds the singleton instance of the Philia_Cashier_Fields class.
     *
     * @var Philia_Cashier_Fields|null
     */
    protected static $_instance = null;

    /**
     * Holds the file path for the log file.
     *
     * @var string
     */
    protected $log_file_path = '';

    /**
     * Singleton instance of the Philia_Cashier_Fields class.
     * Ensures that only one instance of the class exists at any given time.
     * 
     * @return Philia_Cashier_Fields|null Returns the singleton instance of Philia_Cashier_Fields.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Constructor method.
     * Adds actions to initialize settings, create the settings page, enqueue scripts, and handle AJAX requests.
     * 
     * @return void
     */
    public function __construct() {
        add_action('philia_menu', array($this, 'philia_fields_page'), 102);
        add_action('admin_init', array($this, 'philia_fields_init'));
        add_action('init', array($this, 'add_philia_role'));

        // Enqueue necessary JavaScript for AJAX
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
    }

    /**
     * Creates the settings page under the admin menu.
     * 
     * @return void
     */
    public function philia_fields_page() {
        add_submenu_page(
            'philia-api-settings',
            __('Philia Cashier Fields', 'philia'), // Page title
            __('Cashier Fields', 'philia'), // Menu title
            'manage_options', // Capability
            'philia-cashier-fields', // Menu slug
            array($this, 'philia_fields_callback') // Callback function
        );
    }

    /**
     * Initializes the settings and adds settings fields.
     * 
     * @return void
     */
    public function philia_fields_init() {
        register_setting('philia_cashier_fields', 'philia_cashier_fields', array($this, 'sanitize'));

        add_settings_section(
            'philia_cashier_fields_section',
            '',
            array($this, 'philia_cashier_fields_section_callback'),
            'philia-cashier-fields'
        );

        // Four fields
        for( $i = 1; $i <= 4; $i++ )
        {
            add_settings_field(
                'field_'. $i,
                sprintf( __('Field %1$s', 'philia'), $i ),
                array($this, 'field_callback'),
                'philia-cashier-fields',
                'philia_cashier_fields_section',
                ['index' => $i ]
            );
        }
        
        add_settings_field(
            'widget',
            __('Enable Widget', 'philia'),
            array($this, 'widget_callback'),
            'philia-cashier-fields',
            'philia_cashier_fields_section'
        );
        
        add_settings_field(
            'website_url',
            __('Website URL', 'philia'),
            array($this, 'website_callback'),
            'philia-cashier-fields',
            'philia_cashier_fields_section'
        );

        add_settings_field(
            'logo_url',
            __('Logo URL', 'philia'),
            array($this, 'media_upload_callback'),
            'philia-cashier-fields',
            'philia_cashier_fields_section',
            ['field_name' => 'logo_url']
        );

        add_settings_field(
            'login_image_url',
            __('Login Image URL', 'philia'),
            array($this, 'media_upload_callback'),
            'philia-cashier-fields',
            'philia_cashier_fields_section',
            ['field_name' => 'login_image_url']
        );
    }

    /**
     * Displays the Philia API settings page.
     *
     * @return void
     */
    public static function philia_fields_callback() {
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            <form method="post" action="options.php">
                <?php
                settings_fields('philia_cashier_fields');
                do_settings_sections('philia-cashier-fields');
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    /**
     * Outputs the input field for a settings field.
     *
     * @param array $args Arguments containing the field index.
     * @return void
     */
    public function field_callback( $args ) {
        $cashier_fields = get_option('philia_cashier_fields')['fields'];
        ?>
        <input type="text" name="philia_cashier_fields[field_<?php echo esc_attr( $args['index'] ); ?>][title]" placeholder="<?php echo esc_attr__('Field Title', 'philia'); ?>" value="<?php echo esc_attr( $cashier_fields["field_{$args['index']}"]['title'] ?? ''); ?>" size="40" />
        <div class="field-row">
            <h2 class="field-title"><?php echo esc_attr__('Fields:','philia'); ?></h2>
            <?php if( isset( $cashier_fields["field_{$args['index']}"]['values'] ) && !empty( $cashier_fields["field_{$args['index']}"]['values'] ) ) : ?>
                <?php foreach($cashier_fields["field_{$args['index']}"]['values'] as $key => $field ) : ?>
                    <div class="field-section" style="margin-bottom:10px">
                        <input type="text" name="philia_cashier_fields[field_<?php echo esc_attr( $args['index'] ); ?>][key][]" placeholder="<?php echo esc_attr__('Key','philia'); ?>" value="<?php echo esc_attr( $field['key'] ); ?>" size="20" />
                        <input type="text" name="philia_cashier_fields[field_<?php echo esc_attr( $args['index'] ); ?>][value][]" placeholder="<?php echo esc_attr__('Value','philia'); ?>" value="<?php echo esc_attr( $field['value'] ); ?>" size="20" />
                        <?php if ($key === 0) : ?>
                            <button class="button field-add-row" name="field-add-row"><?php echo esc_attr__('Add Row','philia'); ?></button>
                        <?php else : ?>
                            <button class="button field-delete-row" name="field-delete-row"><?php echo esc_attr__('Delete Row','philia'); ?></button>
                        <?php endif; ?>
                    </div>
                <?php endforeach; ?>
            <?php else : ?>
            <div class="field-section" style="margin-bottom:10px">
                <input type="text" name="philia_cashier_fields[field_<?php echo esc_attr( $args['index'] ); ?>][key][]" placeholder="<?php echo esc_attr__('Key','philia'); ?>" value="" size="20" />
                <input type="text" name="philia_cashier_fields[field_<?php echo esc_attr( $args['index'] ); ?>][value][]" placeholder="<?php echo esc_attr__('Value','philia'); ?>" value="" size="20" />
                <button class="button field-add-row" name="field-add-row"><?php echo esc_attr__('Add Row','philia'); ?></button>
            </div>
            <?php endif; ?>
        </div>
        <?php
    }
    
    /**
     * Callback for the widget checkbox field.
     *
     * @return void
     */
    public function widget_callback() {
        $cashier_fields = get_option('philia_cashier_fields');
        $is_widget = isset($cashier_fields['widget']) ? $cashier_fields['widget'] : false;
        ?>
        <label><input type="checkbox" name="philia_cashier_fields[widget]" value="1" <?php checked($is_widget, 1); ?> /> <?php esc_attr_e('Enable Widget',''); ?></label>
        <?php
    }
    
    /**
     * Callback for the widget checkbox field.
     *
     * @return void
     */
    public function website_callback() {
        $cashier_fields = get_option('philia_cashier_fields');
        ?>
        <input type="text" name="philia_cashier_fields[website_url]" value="<?php echo esc_attr($cashier_fields['website_url'] ?? ''); ?>" size="40" />
        <?php
    }

    /**
     * Callback for media upload fields (logo_url and login_image_url).
     *
     * @param array $args Arguments containing the field name.
     * @return void
     */
    public function media_upload_callback($args) {
        $cashier_fields = get_option('philia_cashier_fields');
        $field_name = $args['field_name'];
        $field_value = isset($cashier_fields[$field_name]) ? $cashier_fields[$field_name] : '';
        ?>
        <input type="text" name="philia_cashier_fields[<?php echo esc_attr($field_name); ?>]" id="<?php echo esc_attr($field_name); ?>" value="<?php echo esc_attr($field_value); ?>" class="regular-text" />
        <button class="button philia-media-upload" data-target="<?php echo esc_attr($field_name); ?>"><?php echo esc_attr__('Upload', 'philia'); ?></button>
        <?php
    }

    /**
     * Enqueues necessary JavaScript files for AJAX actions on the settings page.
     *
     * @return void
     */
    public function enqueue_scripts() {
        wp_enqueue_script('philia-cashier-fields', PHILIA_PLUGIN_URL . 'assets/philia-cashier-fields.js', array('jquery'), null, true);
        wp_enqueue_media(); // Enqueue WordPress media scripts

        // Localize script with AJAX URL
        wp_localize_script('philia-cashier-fields', 'philiaFields', array(
            'delete_text' => esc_attr__('Delete Row', 'philia'),
            'upload_title' => esc_attr__('Select or Upload Image', 'philia'),
            'upload_button' => esc_attr__('Use this image', 'philia')
        ));
    }

    /**
     * Callback for the settings section description.
     * 
     * @return void
     */
    public function philia_cashier_fields_section_callback() {
        echo '';
    }

    /**
     * Sanitizes the input from the settings fields.
     * 
     * @param array $input The input from the settings form.
     * @return array The sanitized input values.
     */
    public function sanitize($input) {
        $sanitized_input = array();
    
        // Loop through each field (from field_1 to field_4)
        $fields_input = array();
        for ($i = 1; $i <= 4; $i++) {
            $field_key = "field_{$i}";
            
            // Sanitize the title
            if ( isset( $input[$field_key]['title'] ) ) {
                $fields_input[$field_key]['title'] = sanitize_text_field($input[$field_key]['title']);
            }
            
            if( isset( $input[$field_key]['key'] ) )
            {
                for( $j = 0; $j < count( $input[$field_key]['key'] ); $j++ )
                {
                    if( isset( $input[$field_key]['key'][$j] ) && !empty( $input[$field_key]['key'][$j] ) && isset( $input[$field_key]['value'][$j] ) && !empty( $input[$field_key]['value'][$j] ) )
                    {
                        $fields_input[$field_key]['values'][] = array(
                            'key' => sanitize_text_field( $input[$field_key]['key'][$j] ),
                            'value' => sanitize_text_field( $input[$field_key]['value'][$j] )
                        );
                    }
                }
            }
        }
        
        // Sanitize new fields
        $sanitized_input['fields'] = $fields_input;
        $sanitized_input['widget'] = isset($input['widget']) ? true : false;
        $sanitized_input['website_url'] = isset($input['website_url']) ? esc_url_raw($input['website_url']) : '';
        $sanitized_input['logo_url'] = isset($input['logo_url']) ? esc_url_raw($input['logo_url']) : '';
        $sanitized_input['login_image_url'] = isset($input['login_image_url']) ? esc_url_raw($input['login_image_url']) : '';
    
        return $sanitized_input;
    }
    
    /**
     * Adds a custom "Cashier - Philia" role with specific WooCommerce capabilities.
     *
     * @param void
     * @return void
     */
    public function add_philia_role()
    {
        // Define capabilities
        $capabilities = array(
            'read' => true,
            'manage_woocommerce' => true,
            'manage_products' => true,
        );

        // Add custom role
        add_role(
            'philia_woo_cashier', 
            __('Cashier - Philia', 'philia'), 
            $capabilities
        );
    }
}

if (class_exists('Philia_API\Philia_Cashier_Fields')) {
    Philia_Cashier_Fields::instance();
}