<?php
/**
 * Philia API Cashier Route
 * A REST API route to retrieve cashier fields and users in WooCommerce for the Philia API Plugin.
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Cashier_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the routes for retrieving cashier fields and handling login.
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/cashier/settings', [
            'methods' => 'GET',
            'callback' => [$this, 'get_cashier_settings'],
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);

        register_rest_route('philia/v1', '/user/login', [
            'methods' => 'POST',
            'callback' => [$this, 'handle_login'],
            'permission_callback' => '__return_true'
        ]);
    }

    /**
     * Handles user login and generates an access token if successful.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response Access token and expiration time or error response.
     */
    public function handle_login(\WP_REST_Request $request) {
        $username = $request->get_param('username');
        $password = $request->get_param('password');

        if( empty( $username ) || empty( $password ) ) {
            return new \WP_REST_Response(['error' => 'username and password is required.'], 401);
        }
    
        // Authenticate user
        $user = wp_authenticate($username, $password);
    
        if (is_wp_error($user)) {
            return new \WP_REST_Response(['error' => 'Invalid username or password.'], 401);
        }
    
        // Check user role
        if (!in_array('philia_cashier', $user->roles) && !in_array('administrator', $user->roles) && !in_array('shop_manager', $user->roles)) {
            return new \WP_REST_Response(['error' => 'Unauthorized role.'], 403);
        }
    
        // Generate access token
        $access_token = wp_generate_uuid4();
        $expiration = 10 * HOUR_IN_SECONDS; // Set token expiry to 10 hour
    
        // Store token in transient with expiration
        set_transient('philia_access_token_' . $user->ID, $access_token, $expiration);
    
        return new \WP_REST_Response(['access_token' => $access_token, 'expires_in' => time() + $expiration], 200);
    }

    /**
     * Retrieves a list of cashier fields.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of cashier fields.
     */
    public function get_cashier_settings(\WP_REST_Request $request) {
        $cashier_settings = get_option('philia_cashier_fields');

        if (!is_array($cashier_settings)) {
            return new \WP_REST_Response(['error' => 'No cashier fields found.'], 404);
        }

        $cashier_fields = [];
        foreach ($cashier_settings['fields'] as $key => $field) {
            if (!empty($field['values'])) {
                $cashier_fields[] = [
                    'title' => $field['title'],
                    'id' => $key,
                    'values' => $field['values']
                ];
            }
        }
        
        $final_cashier_fields = [];
        $final_cashier_fields['fields'] = $cashier_fields;
        $final_cashier_fields['widget'] = $cashier_settings['widget'];
        $final_cashier_fields['website_url'] = $cashier_settings['website_url'];
        $final_cashier_fields['logo_url'] = $cashier_settings['logo_url'];
        $final_cashier_fields['login_image_url'] = $cashier_settings['login_image_url'];
        
        return new \WP_REST_Response($final_cashier_fields, 200);
    }

    /**
     * Permission check method to validate access token for secured routes.
     *
     * @param WP_REST_Request $request The REST API request object.
     * @return bool|WP_REST_Response True if access token is valid, otherwise error response.
     */
    public static function permissions_check(\WP_REST_Request $request) {
        $headers = $request->get_headers();
        $auth_header = isset($headers['authorization']) ? $headers['authorization'][0] : '';
    
        if (empty($auth_header) || strpos($auth_header, 'API-Key ') !== 0) {
            return false;
        }
    
        $access_token = str_replace('API-Key ', '', $auth_header);
        $user_id = self::get_user_id_by_token($access_token);
    
        if (!$user_id) {
            return false;
        }
    
        return true;
    }    

    /**
     * Retrieves the user ID by the access token if valid and not expired.
     *
     * @param string $access_token The access token.
     * @return int|false User ID if token is valid, false otherwise.
     */
    private static function get_user_id_by_token($access_token) {
        // Query all users with specific roles
        $user_query = new \WP_User_Query([
            'limit' => -1,
            'role__in' => ['philia_woo_cashier', 'shop_manager', 'administrator']
        ]);

        // Iterate through the results
        foreach ($user_query->get_results() as $user) {
            // Get the transient token for the user
            $stored_token = get_transient('philia_access_token_' . $user->ID);

            // Check if the stored token matches the provided access token
            if ($stored_token && $stored_token === $access_token) {
                // Return the user ID if a match is found
                return $user->ID;
            }
        }

        // Return false if no matching user is found
        return false;
    }

}

/**
 * Registers the Philia cashier REST controller.
 */
function register_philia_cashier_rest_controller() {
    $controller = new Philia_Cashier_REST_Controller();
    $controller->register_routes();
}

add_action('rest_api_init', 'Philia_API\register_philia_cashier_rest_controller');