<?php
/**
 * Philia API Coupon Route
 * A REST API route to retrieve WooCommerce coupons for the Philia API Plugin.
 *
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Coupon_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the routes for coupons.
     */
    public function register_routes() {
        // Route to get a single coupon by code
        register_rest_route('philia/v1', '/coupon/(?P<code>.+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_coupon'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
            'args' => array(
                'code' => array(
                    'required' => true,
                    'validate_callback' => function($param) {
                        return !empty(trim($param)); // Ensures code is provided and not empty
                    }
                )
            )
        ));

        // Route to get all coupons
        register_rest_route('philia/v1', '/coupons', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_all_coupons'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));
    }

    /**
     * Retrieves a single coupon by its code.
     *
     * @param \WP_REST_Request $request The incoming REST request.
     * @return \WP_REST_Response The coupon data or an error response.
     */
    public function get_coupon(\WP_REST_Request $request) {
        $code = sanitize_text_field(trim($request->get_param('code'))); // Sanitize and trim spaces

        $coupon = new \WC_Coupon($code);
        if (!$coupon->get_id()) {
            return new \WP_REST_Response(['message' => 'Coupon not found or invalid'], 404);
        }

        $coupon_data = array(
            'id' => $coupon->get_id(),
            'code' => $coupon->get_code(),
            'discount_type' => $coupon->get_discount_type(),
            'amount' => $coupon->get_amount(),
            'date_expires' => $coupon->get_date_expires(),
            'usage_limit' => $coupon->get_usage_limit(),
            'usage_count' => $coupon->get_usage_count(),
            'minimum_amount' => $coupon->get_minimum_amount(),
            'maximum_amount' => $coupon->get_maximum_amount(),
            'individual_use' => $coupon->get_individual_use(),
            'product_ids' => $coupon->get_product_ids(),
            'exclude_product_ids' => $coupon->get_excluded_product_ids(),
            'usage_restriction' => $coupon->get_usage_limit_per_user()
        );

        return new \WP_REST_Response($coupon_data, 200);
    }

    /**
     * Retrieves all available coupons.
     *
     * @param \WP_REST_Request $request The incoming REST request.
     * @return \WP_REST_Response The list of all coupons.
     */
    public function get_all_coupons(\WP_REST_Request $request) {
        $args = array(
            'post_type' => 'shop_coupon',
            'posts_per_page' => -1,
            'post_status' => 'publish'
        );

        $coupons = get_posts($args);
        $coupon_list = array();

        foreach ($coupons as $coupon_post) {
            $coupon = new \WC_Coupon($coupon_post->post_name);
            $coupon_list[] = array(
                'id' => $coupon->get_id(),
                'code' => $coupon->get_code(),
                'discount_type' => $coupon->get_discount_type(),
                'amount' => $coupon->get_amount(),
                'date_expires' => $coupon->get_date_expires(),
                'usage_limit' => $coupon->get_usage_limit(),
                'usage_count' => $coupon->get_usage_count(),
                'minimum_amount' => $coupon->get_minimum_amount(),
                'maximum_amount' => $coupon->get_maximum_amount(),
                'individual_use' => $coupon->get_individual_use(),
                'product_ids' => $coupon->get_product_ids(),
                'exclude_product_ids' => $coupon->get_excluded_product_ids(),
                'usage_restriction' => $coupon->get_usage_limit_per_user()
            );
        }

        return new \WP_REST_Response($coupon_list, 200);
    }
}

/**
 * Function to register the new coupon routes from the controller.
 */
function register_philia_coupon_rest_controller() {
    $controller = new Philia_Coupon_REST_Controller();
    $controller->register_routes();
}

add_action('rest_api_init', 'Philia_API\register_philia_coupon_rest_controller');