<?php
/**
 * Philia API Customer Route
 * A REST API route to retrieve customers in WooCommerce for the Philia API Plugin.
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @author Philia
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Customer_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the route for retrieving customers
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/customers', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_customers'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/customer/id/(?P<clientID>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_customer_by_id'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/customer/mobile/(?P<mobile>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_customer_by_mobile'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/customer/name/(?P<name>.+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_customer_by_name'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/customer/create', array(
            'methods' => 'POST',
            'callback' => array($this, 'create_customer'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/customer/eligible-coupons/(?P<customer_id>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_eligible_coupons_for_customer'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));
    }

    /**
     * Validates the API key provided in the request headers.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return true|WP_REST_Response True if the API key is valid, otherwise an error response.
     */
    private function validate_api_key($request) {
        $settings = get_option('philia_api_settings');
        
        $headers = $request->get_headers();
        $auth_header = isset($headers['authorization']) ? $headers['authorization'][0] : '';

        if (empty($auth_header) || strpos($auth_header, 'Api-Key ') !== 0) {
            return new \WP_REST_Response(['error' => 'Unauthorized: API key is missing or invalid format.'], 401);
        }

        $api_key = str_replace('Api-Key ', '', $auth_header);

        if (empty($settings) || empty($settings['api_key']) || $api_key !== $settings['api_key']) {
            return new \WP_REST_Response(['error' => 'Unauthorized: Invalid API key.'], 401);
        }

        return true;
    }

    /**
     * Retrieves a list of customers with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function get_customers(\WP_REST_Request $request) {
        
        $page = $request->get_param('page') ? intval($request->get_param('page')) : 1;
        $pageSize = $request->get_param('pageSize') ? intval($request->get_param('pageSize')) : 10;

        // get accounts list from philia api
        $response = Philia_Core_API::send_request('integration/accounts/list/', 'GET', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/accounts/list/' for Accounts List");
        }
        else
        {
            //Philia_API_Log::info("Order created successfully via API - Order ID: " . $order_id);
            //self::add_order_id($order_id);
            $body = wp_remote_retrieve_body( $response );
        }

        /*$args = array(
            'role' => 'customer',
            'number' => $pageSize,
            'offset' => ($page - 1) * $pageSize,
        );
        $users = get_users($args);

        $customers = array_map(function($user) {
            $settings = get_option('philia_api_settings');
            if( isset( $settings ) && !empty( $settings['phone_meta'] ) )
			    $user_phone = get_user_meta( $user->ID, $settings['phone_meta'] , true );
            else
                $user_phone = '';
            return array(
                'id' => $user->ID,
                'first_name' => get_user_meta($user->ID, 'first_name', true),
                'last_name' => get_user_meta($user->ID, 'last_name', true),
                'email' => $user->user_email,
                'mobile' => $user_phone,
            );
        }, $users);

        $customers = [];*/

        return new \WP_REST_Response( json_decode( $body, true ) , 200);
    }

    /**
     * Retrieves a single customer by their client id.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The customer data or an error response.
     */
    public function get_customer_by_id(\WP_REST_Request $request) {
        
        $clientID = intval($request->get_param('clientID'));
        $user = get_user_by('ID', $clientID);

        if ($user && in_array('customer', $user->roles)) {
            $settings = get_option('philia_api_settings');
            if( isset( $settings ) && !empty( $settings['phone_meta'] ) )
			    $user_phone = get_user_meta( $user->ID, $settings['phone_meta'] , true );
            else
                $user_phone = '';

            $customer = array(
                'id' => $user->ID,
                'first_name' => get_user_meta($user->ID, 'first_name', true),
                'last_name' => get_user_meta($user->ID, 'last_name', true),
                'email' => $user->user_email,
                'mobile' => $user_phone,
            );
            return new \WP_REST_Response($customer, 200);
        }

        return new \WP_REST_Response(['message' => 'Customer not found'], 404);
    }

    /**
     * Retrieves a customer by their mobile number.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The customer data or an error response.
     */
    public function get_customer_by_mobile(\WP_REST_Request $request) {
        $mobile = $request->get_param('mobile');
        
        // Validate mobile parameter
        if (empty($mobile)) {
            return new \WP_REST_Response(['message' => 'Mobile number is required'], 400);
        }
    
        // Fetch the settings to get the phone_meta key
        $page = $request->get_param('page') ? intval($request->get_param('page')) : 1;
        $pageSize = $request->get_param('pageSize') ? intval($request->get_param('pageSize')) : 10;

        // get accounts list from philia api
        $response = Philia_Core_API::send_request( 'integration/accounts/list/?mobile=' . $mobile , 'GET', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/accounts/list/' for Accounts List - Search by mobile");
        }
        else
        {
            //Philia_API_Log::info("Order created successfully via API - Order ID: " . $order_id);
            //self::add_order_id($order_id);
            $body = wp_remote_retrieve_body( $response );
        }

        return new \WP_REST_Response( json_decode( $body, true ) , 200);
        /*$settings = get_option('philia_api_settings');
        if (isset($settings) && !empty($settings['phone_meta'])) {
            $args = array(
                'meta_query' => array(
                    array(
                        'key'     => esc_attr($settings['phone_meta']),
                        'value'   => $mobile,
                        'compare' => 'LIKE' // Use 'LIKE' for partial matches
                    )
                ),
                'role' => 'customer',
                'orderby' => 'registered', // Order by ID
                'order'   => 'ASC', // In ascending order
            );
            
            // Create a new WP_User_Query
            $user_query = new \WP_User_Query($args);
            $users = $user_query->get_results();
    
            // Handle the results
            if (!empty($users)) {
                $customer_list = [];
    
                // Loop through users and prepare the response
                foreach ($users as $user) {
                    // Retrieve user's phone number from meta
                    $user_phone = get_user_meta($user->ID, $settings['phone_meta'], true);
                    
                    $customer_list[] = array(
                        'id' => $user->ID,
                        'first_name' => get_user_meta($user->ID, 'first_name', true),
                        'last_name' => get_user_meta($user->ID, 'last_name', true),
                        'email' => $user->user_email,
                        'mobile' => $user_phone,
                    );
                }
    
                // Return the mobile number as %mobile% along with the customers list
                return new \WP_REST_Response( $customer_list, 200 );
            }
        }
    
        return new \WP_REST_Response(['message' => 'Customer not found'], 404);*/
    }

    /**
     * Retrieves a list of customers by their first or last name.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response A list of matching customers or an error response.
     */
    public function get_customer_by_name(\WP_REST_Request $request) {
        
        $name = $request->get_param('name');

        if( empty( $name ) )
            return new \WP_REST_Response(['error' => 'Customer name is a required field.'], 400);
        
        // Sanitize the input
        $name = sanitize_text_field( trim( urldecode($name) ) );
    
        // Prepare WP_User_Query arguments
        $args = array(
            'role' => 'customer',
            'meta_query' => array(
                'relation' => 'OR',
                array(
                    'key' => 'first_name',
                    'value' => $name,
                    'compare' => 'LIKE' // Match first name
                ),
                array(
                    'key' => 'last_name',
                    'value' => $name,
                    'compare' => 'LIKE' // Match last name
                ),
            ),
            'number' => -1, // Get all matching users
            'orderby' => 'registered', // Order by ID
            'order'   => 'ASC', // In ascending order
        );
    
        // Create a new WP_User_Query
        $user_query = new \WP_User_Query($args);
        $users = $user_query->get_results();
    
        // Prepare the customer list
        $customer_list = array_map(function($user) {
            $settings = get_option('philia_api_settings');
            $user_phone = isset($settings) && !empty($settings['phone_meta']) 
                ? get_user_meta($user->ID, $settings['phone_meta'], true) 
                : '';
    
            return array(
                'id' => $user->ID,
                'first_name' => get_user_meta($user->ID, 'first_name', true),
                'last_name' => get_user_meta($user->ID, 'last_name', true),
                'email' => $user->user_email,
                'mobile' => $user_phone,
            );
        }, $users);
    
        // Return the customer list or a not found message
        if (!empty($customer_list)) {
            return new \WP_REST_Response(array_values($customer_list), 200);
        }
    
        return new \WP_REST_Response(['message' => 'Customer not found'], 404);
    }

    /**
     * Creates a new customer with first name, last name, and mobile number.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response Response with success message or error.
     */
    public function create_customer(\WP_REST_Request $request) {
        
        $first_name = sanitize_text_field($request->get_param('firstName'));
        $last_name = sanitize_text_field($request->get_param('lastName'));
        $mobile = sanitize_text_field($request->get_param('mobile'));

        // Validate required fields
        if (empty($first_name) || empty($last_name) || empty($mobile)) {
            return new \WP_REST_Response(['error' => 'firstName, lastName, and mobile are required fields.'], 400);
        }

        // Check if a user with the same mobile already exists
        $settings = get_option('philia_api_settings');
        $phone_meta_key = isset($settings['phone_meta']) ? $settings['phone_meta'] : '';
        $user_query = new \WP_User_Query(array(
            'meta_key' => $phone_meta_key,
            'meta_value' => $mobile,
            'role' => 'customer',
            'orderby' => 'registered', // Order by ID
            'order'   => 'ASC', // In ascending order
        ));
        
        if (!empty($user_query->get_results())) {
            return new \WP_REST_Response(['error' => 'A customer with this mobile number already exists.'], 409);
        }

        // Create new user with the customer role
        $user_id = wp_insert_user(array(
            'role' => 'customer',
            'first_name' => $first_name,
            'last_name' => $last_name,
            'user_email' => Philia_User_API::generate_random_email_address(),
            'user_login' => strtolower( wp_generate_password(16, false) ),
            'user_pass' => wp_generate_password(),
        ));

        if (is_wp_error($user_id)) {
            return new \WP_REST_Response(['error' => 'Failed to create customer.'], 500);
        }

        // Save mobile number in specified meta key
        if ($phone_meta_key) {
            update_user_meta($user_id, $phone_meta_key, $mobile);
        }

        return new \WP_REST_Response(['success' => 'Customer created successfully.', 'user_id' => $user_id], 201);
    }

    /**
     * Retrieves a list of coupons that a specific customer is eligible to use.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response A list of eligible coupons for the customer or an error response.
     */
    public function get_eligible_coupons_for_customer(\WP_REST_Request $request)
    {
        // Validate the API key
        $auth = $this->validate_api_key($request);
        if (is_wp_error($auth)) return $auth;

        // Get the user ID from the request
        $user_id = intval($request->get_param('customer_id'));
        if (!$user_id) {
            return new \WP_REST_Response(['error' => 'Customer ID is required'], 400);
        }

        // Retrieve all active coupons
        $args = array(
            'post_type' => 'shop_coupon',
            'posts_per_page' => -1,
            'post_status' => 'publish',
        );
        $coupons = get_posts($args);

        $eligible_coupons = [];
        foreach ($coupons as $coupon_post) {
            $coupon = new \WC_Coupon($coupon_post->post_name);

            // Check if coupon is restricted to specific user roles or usage limits
            if (! self::is_coupon_eligible_for_user($coupon, $user_id)) {
                continue;
            }

            $eligible_coupons[] = [
                'id' => $coupon->get_id(),
                'code' => $coupon->get_code(),
                'description' => $coupon->get_description(),
                'discount_type' => $coupon->get_discount_type(),
                'amount' => $coupon->get_amount(),
                'date_expires' => $coupon->get_date_expires() ? $coupon->get_date_expires()->date('Y-m-d') : null,
                'usage_limit' => $coupon->get_usage_limit(),
                'usage_count' => $coupon->get_usage_count(),
                'minimum_amount' => $coupon->get_minimum_amount(),
                'maximum_amount' => $coupon->get_maximum_amount(),
                'individual_use' => $coupon->get_individual_use(),
                'product_ids' => $coupon->get_product_ids(),
                'exclude_product_ids' => $coupon->get_excluded_product_ids(),
                'usage_restriction' => $coupon->get_usage_limit_per_user()
            ];
        }

        return new \WP_REST_Response($eligible_coupons, 200);
    }

    /**
     * Checks if a coupon is eligible for a specific user.
     *
     * @param WC_Coupon $coupon The WooCommerce coupon object.
     * @param int $user_id The user ID to check eligibility for.
     * @return bool True if the coupon is eligible for the user, otherwise false.
     */
    private static function is_coupon_eligible_for_user($coupon, $user_id)
    {
        $usage_limit = $coupon->get_usage_limit();
        $usage_count = $coupon->get_usage_count();

        // Check if coupon usage limit has been reached
        if ($usage_limit && $usage_count >= $usage_limit) {
            return false;
        }

        // Check if the user has already used this coupon
        if ($coupon->get_used_by() && in_array($user_id, $coupon->get_used_by())) {
            return false;
        }

        // Check if the coupon has expired
        $expiration_date = $coupon->get_date_expires();
        if ($expiration_date && $expiration_date < new \WC_DateTime()) {
            return false;
        }

        // Add more checks here if you have specific usage restrictions like minimum spend, products, etc.
        return true;
    }
}

/**
 * Function to register our new routes from the controller.
 */
function register_philia_customer_rest_controller() {
	$controller = new Philia_Customer_REST_Controller();
	$controller->register_routes();
}

add_action( 'rest_api_init', 'Philia_API\register_philia_customer_rest_controller' );