<?php
/**
 * Philia API Invoice Route
 * A REST API route to retrieve orders in WooCommerce for the Philia API Plugin.
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @author Philia
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Invoice_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the route for retrieving product categories.
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/invoice/(?P<id>\d+)', [
            'methods'  => 'GET',
            'callback' => array($this, 'get_invoice'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);

        register_rest_route('philia/v1', '/invoices', [
            'methods'  => 'GET',
            'callback' => array($this, 'get_invoices'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);

        register_rest_route('philia/v1', '/invoice/calculation', [
            'methods' => \WP_REST_Server::CREATABLE,
            'callback' => array($this, 'calculate_invoice'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);

        register_rest_route('philia/v1', '/invoice/create', [
            'methods'  => 'POST',
            'callback' => array($this, 'create_invoice'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);
    }

    /**
     * Retrieves a list of invoices with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function get_invoices(\WP_REST_Request $request) {
        $page = $request->get_param('page') ? intval($request->get_param('page')) : 1;
        $pageSize = $request->get_param('pageSize') ? intval($request->get_param('pageSize')) : 10;
    
        $args = [
            'orderby' => 'date',
            'order'   => 'DESC',
            'limit'  => $pageSize,
            'offset' => ($page - 1) * $pageSize,
        ];
        $orders = wc_get_orders($args);
    
        $data = [];
    
        foreach ($orders as $order) {
            if ( $order instanceof \Automattic\WooCommerce\Admin\Overrides\OrderRefund)
            {
                $order = wc_get_order( $order->get_parent_id() );
            }
            else
            {
                $order = wc_get_order( $order->get_id() );
            }

            $products = [];
            if( !empty( $order->get_items() ) )
            {
                foreach( $order->get_items() as $item )
                {
                    $subtotal = $item->get_subtotal();
                    $total = $item->get_total();
                    $discount = $subtotal - $total;
                    $products[] = (object) array(
                        'id'            => $item->get_product_id(),
                        'name'          => $item->get_name(),
                        'price'         => $item->get_total() / $item->get_quantity(),
                        'quantity'      => $item->get_quantity(),
                        'totalPrice'    => $item->get_total(),
                        'totalDiscount' => $discount,
                    );
                }
            }

            $data[] = [
                'id'           => $order->get_id(),
                'invoice_id'   => $order->get_id(),
                'first_name'   => $order->get_billing_first_name(),
                'last_name'    => $order->get_billing_last_name(),
                'mobile'       => $order->get_billing_phone(),
                'date'         => $order->get_date_created()->date('c'),
                'totalAmount'  => $order->get_total(),
                'products'     => $products,
            ];
        }
    
        return new \WP_REST_Response($data, 200);
    }

    /**
     * Retrieves a single customer by their client id.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The customer data or an error response.
     */
    public function get_invoice(\WP_REST_Request $request) {
        $order_id = intval($request['id']);
        $order = wc_get_order($order_id);
        if ( $order instanceof \Automattic\WooCommerce\Admin\Overrides\OrderRefund)
        {
            $order = wc_get_order( $order->get_parent_id() );
        }
        else
        {
            $order = wc_get_order( $order->get_id() );
        }

        if (!$order) {
            return new \WP_REST_Response(['error' => 'Invoice not found.'], 404);
        }

        $products = [];
        if( !empty( $order->get_items() ) )
        {
            foreach( $order->get_items() as $item )
            {
                $subtotal = $item->get_subtotal();
                $total = $item->get_total();
                $discount = $subtotal - $total;
                $products[] = (object) array(
                    'id'            => $item->get_product_id(),
                    'name'          => $item->get_name(),
                    'price'         => $item->get_total() / $item->get_quantity(),
                    'quantity'      => $item->get_quantity(),
                    'totalPrice'    => $item->get_total(),
                    'totalDiscount' => $discount,
                );
            }
        }

        $data = [
            'first_name'      => $order->get_billing_first_name(),
            'last_name'       => $order->get_billing_last_name(),
            'date'            => $order->get_date_created()->date('c'),
            'mobile'          => $order->get_billing_phone(),
            'invoice_id'      => $order->get_id(),
            'totalAmount'     => $order->get_total(),
            'totalDiscount'   => $order->get_discount_total(),
            'deliveryAmount'  => $order->get_shipping_total(),
            'note'            => $order->get_customer_note(),
            'products'        => $products,
        ];

        return new \WP_REST_Response( $data, 200);
    }

    public function calculate_invoice(\WP_REST_Request $request)
    {
        $products = $request->get_param('products');
        $coupon_code = $request->get_param('coupon');
        $customer_id = intval($request->get_param('customerId'));

        if( empty( $products ) )
            return new \WP_REST_Response(['error' => 'Products is a required field.'], 400);

        if( empty( $customer_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        $products_ids = array_column( $products, 'id' );

        $api_data = array(
            'product_ids' => $products_ids,
            'user_id' => $customer_id
        );

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/list/products' , 'POST', $api_data, 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/list/products' for Rewards Products List - Customer ID: " . $customer_id );
        }
        else
        {
            $rewards_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
        }
    
        $totalAmount = 0;
        $totalDiscount = 0;
        $processed_products = [];
    
        // Loop through each product to calculate totals
        foreach ($products as $product_data) {
            $product_id = absint( $product_data['id'] );
            $quantity = absint( $product_data['quantity'] );
            $reward_id = absint( $product_data['rewardId'] );
            $product = wc_get_product($product_id);
    
            if ($product && $quantity > 0)
            {
                $product_name = $product->get_name();
                $regular_price = floatval($product->get_regular_price());
                // calc discount
                if( !empty( $reward_id ) )
                {
                    // find reward id and calculate product price
                    $founded_reward_info = array_filter($rewards_result['items'], function ($reward) use ($reward_id) {
                        return $reward['id'] === $reward_id;
                    });

                    //if( $founded_reward_info[''] == '')
                    // if flag == 0 use regular price else sale price
                    
                    // $sale_price = $product->get_sale_price() ? floatval($product->get_sale_price()) : $regular_price;
                    // $product_price = $sale_price;  // Use sale price if available, else regular price
                }
                else
                {
                    $sale_price = $product->get_sale_price() ? floatval($product->get_sale_price()) : $regular_price;
                    $product_price = $sale_price;  // Use sale price if available, else regular price
                }

                $product_total_price = $regular_price * $quantity;
                $product_discount = ($regular_price - $product_price) * $quantity; // Discount per unit
    
                $totalAmount += $product_total_price;
                $totalDiscount += $product_discount;
    
                // Append product details to the processed list
                $processed_products[] = (object) array(
                    'id'          => $product_id,
                    'name'        => $product_name,
                    'price'       => $regular_price,
                    'quantity'    => $quantity,
                    'totalPrice'  => $product_total_price,
                    'totalDiscount' => $product_discount,
                );
            }
        }
    
        // Check if the coupon is valid for the customer
        if ($coupon_code) {
            $coupon = new \WC_Coupon($coupon_code);
    
            if (!$coupon->is_valid()) {
                return new \WP_REST_Response(['error' => 'Invalid coupon code'], 400);
            }
    
            if (! Philia_Customer_REST_Controller::is_coupon_eligible_for_user($coupon, $customer_id)) {
                return new \WP_REST_Response(['error' => 'Coupon not eligible for this customer'], 403);
            }
    
            // Calculate the discount based on the coupon type
            if ($coupon->get_discount_type() === 'fixed_cart') {
                $totalDiscount += floatval($coupon->get_amount());
            } elseif ($coupon->get_discount_type() === 'percent') {
                $totalDiscount += ($totalAmount * (floatval($coupon->get_amount()) / 100));
            }
        }
    
        // Final calculation to ensure no negative total
        $totalAmountAfterDiscount = max(0, $totalAmount - $totalDiscount);
    
        // Construct the response
        $basket = [
            'totalAmount'    => $totalAmountAfterDiscount,
            'totalDiscount'  => $totalDiscount,
            'products'       => $processed_products,
        ];
    
        return new \WP_REST_Response($basket, 200);
    }

    /**
     * Retrieves a list of customers by their first or last name.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response A list of matching customers or an error response.
     */
    public function create_invoice(\WP_REST_Request $request) {
        // Retrieve and validate parameters from the request
        $customer_id = $request->get_param('customerId');
        $total_amount = $request->get_param('totalAmount');
        $total_discount = $request->get_param('totalDiscount');
        $delivery_amount = $request->get_param('deliveryAmount');
        $products = $request->get_param('products');
        $cashier_fields = $request->get_param('cashierFields');
        $coupon = $request->get_param('coupon');
    
        // Create a new WooCommerce order
        $order = wc_create_order();
    
        // Set the customer ID for the order
        if ($customer_id) {
            $order->set_customer_id($customer_id);
            $customer_info = get_userdata($customer_id);
            // Set billing details
            if ($customer_info) {
                $order->set_billing_first_name($customer_info->first_name);
                $order->set_billing_last_name($customer_info->last_name);
                $settings = get_option('philia_api_settings');
                if( !empty( $settings['phone_meta'] ) )
                {
                    $user_phone = get_user_meta( $customer_info->ID, $settings['phone_meta'] , true );
                    if( !empty( $user_phone ) )
                    {
                        $order->set_billing_phone( $user_phone );
                    }
                }
            }
        }
    
        // Loop through each product in the request and add to the order
        foreach ($products as $product_data)
        {
            $product_id = absint( $product_data['id'] );
            $quantity = absint( $product_data['quantity'] );
            $total_price = $product_data['totalPrice'];
            
            $product = wc_get_product($product_id);
            if ($product) {
                // Add product to the order with the specified quantity
                $item_id = $order->add_product($product, $quantity);
                
                // Set line item total price and discount
                $item = $order->get_item($item_id);

                if (isset($product_data['totalDiscount']) && !empty($product_data['totalDiscount']) && $product_data['totalDiscount'] > 0) {
                    // Calculate the subtotal (original price before discount)
                    $subtotal = $product_data['totalPrice'] - $product_data['totalDiscount'];
                    $item->set_subtotal($subtotal); // Set the original price
                    $item->set_total($subtotal); // Set the discounted price
                }
                else
                {
                    $item->set_subtotal($product_data['totalPrice']);
                    $item->set_total($product_data['totalPrice']);
                }

                $item->save();
            }
        }
    
        // Apply coupon if provided
        if ($coupon) {
            $order->apply_coupon($coupon);
        }
    
        // Set order total amounts
        $order->set_total($total_amount, 'cart');
        $order->set_total($total_discount, 'discount');
        $order->set_total($delivery_amount, 'shipping');
    
        // Optionally add custom cashier fields to order meta
        if ($cashier_fields && is_array($cashier_fields)) {
            foreach ($cashier_fields as $field_key => $field_value) {
                $order->update_meta_data('cashier_' . $field_key, $field_value);
            }
        }
    
        // Calculate totals and save the order
        $order->calculate_totals();
    
        // Set the order status to completed
        $order->update_status('completed', 'Order automatically marked as completed.');
    
        return new \WP_REST_Response([
            'invoice_id' => $order->get_id(),
            'status' => 'Order created and marked as completed successfully'
        ], 201);
    }

    /**
     * Permission check method.
     * Checks if the current user has the required capability to access the endpoint.
     *
     * @param WP_REST_Request $request The REST API request object.
     * @return bool True if the current user can manage options, otherwise false.
     */
    public function permissions_check(\WP_REST_Request $request) {
        return current_user_can('manage_options');
    }
}

/**
 * Function to register our new routes from the controller.
 */
function register_philia_invoice_rest_controller() {
	$controller = new Philia_Invoice_REST_Controller();
	$controller->register_routes();
}

add_action( 'rest_api_init', 'Philia_API\register_philia_invoice_rest_controller' );