<?php
/**
 * Philia API Product Category Route
 * A REST API route to retrieve products categories in WooCommerce for the Philia API Plugin.
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @author Philia
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Product_Category_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the route for retrieving product categories.
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/products-categories', [
            [
                'methods' => 'GET',
                'callback' => [$this, 'get_product_categories'],
                'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
            ],
        ]);
    }

    /**
     * Retrieves WooCommerce product categories if the API key is valid.
     * 
     * @param WP_REST_Request $request The REST API request object.
     * @return WP_REST_Response A response object containing the product categories data or an error message.
     */
    public function get_product_categories(\WP_REST_Request $request)
    {
        $args = [
            'taxonomy' => 'product_cat',
            'hide_empty' => false,
        ];

        $categories = get_terms($args);

        if (is_wp_error($categories)) {
            return new \WP_REST_Response(['error' => 'Unable to retrieve product categories.'], 500);
        }

        $data = array_map(function($category) {
            return [
                'id' => $category->term_id,
                'name' => $category->name,
            ];
        }, $categories);

        return new \WP_REST_Response($data, 200);
    }
}

/**
 * Function to register our new routes from the controller.
 */
function register_philia_product_category_rest_controller() {
	$controller = new Philia_Product_Category_REST_Controller();
	$controller->register_routes();
}

add_action( 'rest_api_init', 'Philia_API\register_philia_product_category_rest_controller' );