<?php
/**
 * Philia API Products Route
 * A REST API route to retrieve products in WooCommerce for the Philia API Plugin.
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @author Philia
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Product_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the route for retrieving products.
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/products', [
            [
                'methods'  => \WP_REST_Server::READABLE,
                'callback' => [$this, 'get_products'],
                'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
            ],
        ]);

        register_rest_route('philia/v1', '/products/popular', [
            'methods' => \WP_REST_Server::READABLE,
            'callback' => [$this, 'get_popular_products'],
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ]);

        register_rest_route('philia/v1', '/products/customer-popular/(?P<customer_id>\d+)', [
            'methods' => \WP_REST_Server::READABLE,
            'callback' => [$this, 'get_popular_products_for_customer'],
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
            'args' => [
                'customer_id' => [
                    'required' => true,
                    'validate_callback' => function($param) {
                        return is_numeric($param);
                    }
                ]
            ],
        ]);
    }

    /**
     * Retrieves WooCommerce products if the API key is valid.
     * 
     * @param WP_REST_Request $request The REST API request object.
     * @return WP_REST_Response A response object containing the product categories data or an error message.
     */
    public function get_products(\WP_REST_Request $request)
    {
        $customer_id = intval($request->get_param('customerId'));

        if( empty( $customer_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        // Get all WooCommerce products
        $args = [
            'status' => 'publish',
            'limit'  => -1 // Retrieve all products; adjust as needed
        ];

        $category_id = $request->get_param('category');

        if( !empty( $category_id ) && absint( $category_id ) > 0 )
        {
            $args['product_category_id'][] = $category_id;
        }

        $products = wc_get_products($args);

        $product_ids = [];
        if( !empty( $products ) )
        {
            foreach($products as $product)
                $product_ids[] = $product->get_id();
        }

        $api_data = array(
            'product_ids' => $products_ids,
            'user_id' => $customer_id
        );

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/list/products' , 'POST', $api_data, 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/list/products' for Rewards Products List - Customer ID: " . $customer_id );
        }
        else
        {
            $rewards_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
        }

        $product_data = array_map(function($product) {
            $image_url = wp_get_attachment_image_src( get_post_thumbnail_id( $product->get_id() ) , 'single-post-thumbnail' );
            $product_id = $product->get_id();
            $product_rewards = array_filter($rewards_result['items'], function ($reward) use ($product_id) {
                return in_array( $product_id, $reward['product_ids'] );
            });
            
            return [
                'id'          => $product->get_id(),
                'name'        => $product->get_name(),
                'image'       => ( $image_url[0] ) ? $image_url[0] : '',
                'categories'  => $product->get_category_ids(),
                'price'       => $product->get_price(),
                'description' => $product->get_description(),
                'stock'       => $product->get_stock_quantity(),
                'rewards'     => $product_rewards,
            ];
        }, $products);

        return new \WP_REST_Response($product_data, 200);
    }

    /**
     * Retrieves popular products across the entire shop.
     *
     * @return WP_REST_Response A list of popular products across the shop.
     */
    public function get_popular_products() {
        $args = [
            'status' => 'publish',
            'limit' => 10, // Change this limit as needed
            'orderby' => 'popularity',
        ];

        $products = wc_get_products($args);
        $product_data = array_map(function($product) {
            $image_url = wp_get_attachment_image_src( get_post_thumbnail_id( $product->get_id() ) , 'single-post-thumbnail' );
            return [
                'id'          => $product->get_id(),
                'name'        => $product->get_name(),
                'image'       => ( $image_url[0] ) ? $image_url[0] : '',
                'categories'  => $product->get_category_ids(),
                'price'       => $product->get_price(),
                'description' => $product->get_description(),
                'stock'       => $product->get_stock_quantity(),
            ];
        }, $products);

        return new \WP_REST_Response($product_data, 200);
    }

    /**
     * Retrieves popular products based on a specific customer's purchase history.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response A list of popular products for the customer.
     */
    public function get_popular_products_for_customer($request) {
        $customer_id = intval($request['customer_id']);

        // Fetch all orders for the customer
        $orders = wc_get_orders([
            'customer_id' => $customer_id,
            'limit' => -1,
        ]);

        $product_counts = [];

        // Count product purchases per order
        foreach ($orders as $order) {
            foreach ($order->get_items() as $item) {
                $product_id = $item->get_product_id();
                if (isset($product_counts[$product_id])) {
                    $product_counts[$product_id] += $item->get_quantity();
                } else {
                    $product_counts[$product_id] = $item->get_quantity();
                }
            }
        }

        // Sort products by purchase count
        arsort($product_counts);

        // Retrieve popular products based on purchase count
        $products = array_map(function($product_id) {
            $product = wc_get_product($product_id);
            $image_url = wp_get_attachment_image_src( get_post_thumbnail_id( $product->get_id() ) , 'single-post-thumbnail' );
            return [
                'id'          => $product->get_id(),
                'name'        => $product->get_name(),
                'image'       => ( $image_url[0] ) ? $image_url[0] : '',
                'categories'  => $product->get_category_ids(),
                'price'       => $product->get_price(),
                'description' => $product->get_description(),
                'stock'       => $product->get_stock_quantity(),
            ];
        }, array_keys($product_counts));

        return new \WP_REST_Response($products, 200);
    }
}

/**
 * Function to register our new routes from the controller.
 */
function register_philia_product_rest_controller() {
	$controller = new Philia_Product_REST_Controller();
	$controller->register_routes();
}

add_action( 'rest_api_init', 'Philia_API\register_philia_product_rest_controller' );