<?php
/**
 * Philia API Reward Route
 * A REST API route to retrieve rewards in Philia for cashier
 * 
 * @package Philia
 * @subpackage Philia_API\Rest_Routes
 * @author Philia
 * @license MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
}

class Philia_Reward_REST_Controller extends \WP_REST_Controller {

    /**
     * Registers the route for retrieving product categories.
     */
    public function register_routes() {
        register_rest_route('philia/v1', '/rewards/list/(?P<customer_id>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_rewards_by_userid'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/rewards/detail/(?P<reward_id>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_reward_by_id'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/rewards/claim/', array(
            'methods' => 'POST',
            'callback' => array($this, 'claim_reward_by_userid'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/rewards/burn/code/', array(
            'methods' => 'POST',
            'callback' => array($this, 'burn_reward_code'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));

        register_rest_route('philia/v1', '/rewards/list/claimed/(?P<customer_id>\d+)', array(
            'methods' => 'GET',
            'callback' => array($this, 'get_claimed_rewards_by_userid'),
            'permission_callback' => ['Philia_API\Philia_Cashier_REST_Controller', 'permissions_check'],
        ));
    }

    /**
     * Validates the API key provided in the request headers.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return true|WP_REST_Response True if the API key is valid, otherwise an error response.
     */
    private function validate_api_key($request) {
        $settings = get_option('philia_api_settings');
        
        $headers = $request->get_headers();
        $auth_header = isset($headers['authorization']) ? $headers['authorization'][0] : '';

        if (empty($auth_header) || strpos($auth_header, 'Api-Key ') !== 0) {
            return new \WP_REST_Response(['error' => 'Unauthorized: API key is missing or invalid format.'], 401);
        }

        $api_key = str_replace('Api-Key ', '', $auth_header);

        if (empty($settings) || empty($settings['api_key']) || $api_key !== $settings['api_key']) {
            return new \WP_REST_Response(['error' => 'Unauthorized: Invalid API key.'], 401);
        }

        return true;
    }

    /**
     * Retrieves a list of customers with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function get_rewards_by_userid(\WP_REST_Request $request) {
        
        $customer_id = $request->get_param('customer_id');
        $page = $request->get_param('page') ? absint($request->get_param('page')) : 1;

        if( empty( $customer_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/list/' . $customer_id . '/?page=' . $page , 'GET', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/list/' for Rewards List - Customer ID: " . $customer_id );
        }
        else
        {
            $api_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
            return new \WP_REST_Response( $api_result , wp_remote_retrieve_response_code( $response ) );
        }
    }

    /**
     * Retrieves a list of customers with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function get_reward_by_id(\WP_REST_Request $request) {
        
        $reward_id = $request->get_param('reward_id');

        if( empty( $reward_id ) )
            return new \WP_REST_Response(['error' => 'Reward ID is a required field.'], 400);

        // get reward details from philia api
        $response = Philia_Core_API::send_request('integration/rewards/detail/' . $reward_id , 'GET', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/detail/' for Rewards Detail - Reward ID: " . $reward_id );
        }
        else
        {
            $api_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
            return new \WP_REST_Response( $api_result , wp_remote_retrieve_response_code( $response ) );
        }
    }

    /**
     * Retrieves a list of customers with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function claim_reward_by_userid(\WP_REST_Request $request) {
        
        $reward_id = absint( $request->get_param('reward_id') );
        $user_id = absint( $request->get_param('user_id') );

        if( empty( $reward_id ) )
            return new \WP_REST_Response(['error' => 'Reward ID is a required field.'], 400);

        if( empty( $user_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/claim/' , 'POST', [ 'user_id' => $user_id, 'reward_id' => $reward_id ], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/claim/' for Claim Reward of Customer ID: " . $user_id );
        }
        else
        {
            $api_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
            return new \WP_REST_Response( $api_result , wp_remote_retrieve_response_code( $response ) );
        }
    }

    /**
     * Burn Reward Code.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function burn_reward_code(\WP_REST_Request $request) {
        
        $code = $request->get_param('code');
        $reward_id = $request->get_param('reward_id');
        $user_id = $request->get_param('customer_id');

        if( empty( $code ) )
            return new \WP_REST_Response(['error' => 'Reward Code is a required field.'], 400);

        if( empty( $reward_id ) )
            return new \WP_REST_Response(['error' => 'Reward ID is a required field.'], 400);

        if( empty( $user_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/burn/code/?code=' . $code.'&user_id='. $user_id. '&reward_id=' . $reward_id , 'POST', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/burn/code/' for Burn Reward of Customer ID: " . $user_id . " - Reward ID: " . $reward_id );
        }
        else
        {
            $api_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
            return new \WP_REST_Response( $api_result , wp_remote_retrieve_response_code( $response ) );
        }
    }

    /**
     * Retrieves a list of customers with pagination.
     *
     * @param WP_REST_Request $request The incoming REST request.
     * @return WP_REST_Response The list of customers or an error response.
     */
    public function get_claimed_rewards_by_userid(\WP_REST_Request $request) {
        
        $customer_id = $request->get_param('customer_id');
        $page = $request->get_param('page') ? absint($request->get_param('page')) : 1;

        if( empty( $customer_id ) )
            return new \WP_REST_Response(['error' => 'Customer ID is a required field.'], 400);

        // get rewards list from philia api
        $response = Philia_Core_API::send_request('integration/rewards/list/claimed/' . $customer_id . '/?page=' . $page , 'GET', [], 2);

        if ( is_wp_error( $response ) )
        {
            Philia_API_Log::error("Error in 'integration/rewards/list/claimed/' for Claimed Rewards List - Customer ID: " . $customer_id );
        }
        else
        {
            $api_result = json_decode( wp_remote_retrieve_body( $response, true ) , true );
            return new \WP_REST_Response( $api_result , wp_remote_retrieve_response_code( $response ) );
        }
    }
}

/**
 * Function to register our new routes from the controller.
 */
function register_philia_reward_rest_controller() {
	$controller = new Philia_Reward_REST_Controller();
	$controller->register_routes();
}

add_action( 'rest_api_init', 'Philia_API\register_philia_reward_rest_controller' );