<?php
/**
 * Philia API Settings
 * The class responsible for managing the Philia API settings in WordPress.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2025/03/04 12:57:42
 * License: MIT
 */

namespace Philia_API;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

class Philia_API_Settings {
    
    /**
     * Holds the singleton instance of the Philia_API_Settings class.
     *
     * @var Philia_API_Settings|null
     */
    protected static $_instance = null;

    /**
     * Holds the file path for the log file.
     *
     * @var string
     */
    protected $log_file_path = '';

    /**
     * Singleton instance of the Philia_API_Settings class.
     * Ensures that only one instance of the class exists at any given time.
     * 
     * @return Philia_API_Settings|null Returns the singleton instance of Philia_API_Settings.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Constructor method.
     * Adds actions to initialize settings, create the settings page, enqueue scripts, and handle AJAX requests.
     * 
     * @return void
     */
    public function __construct() {
        add_action('admin_menu', array($this, 'philia_api_settings_page'));
        add_action('admin_init', array($this, 'philia_api_settings_init'));
        add_action('admin_notices', array($this, 'philia_settings_saved_notice'));

        // Enqueue necessary JavaScript for AJAX
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));

        // Handle AJAX requests
        add_action('wp_ajax_philia_api_test_connection', array($this, 'ajax_philia_api_test_connection'));
        add_action('wp_ajax_clear_philia_api_log', array($this, 'ajax_clear_philia_api_log'));
        add_action('wp_ajax_clear_philia_data', array($this, 'ajax_clear_philia_data'));
    }

    /**
     * Creates the settings page under the admin menu.
     * 
     * @return void
     */
    public function philia_api_settings_page() {
        add_submenu_page(
            'philia-api-settings',
            __('Philia API Settings', 'philia'), // Page title
            __('Settings', 'philia'), // Menu title
            'manage_options', // Capability
            'philia-api-settings', // Menu slug
            array($this, 'philia_api_settings_callback') // Callback function
        );
    }

    /**
     * Initializes the settings and adds settings fields.
     * 
     * @return void
     */
    public function philia_api_settings_init() {
        register_setting('philia_api_settings', 'philia_api_settings', array($this, 'sanitize'));

        add_settings_section(
            'philia_api_settings_section',
            '',
            array($this, 'philia_api_settings_section_callback'),
            'philia-api-settings'
        );

        // API URL
        add_settings_field(
            'api_url',
            __('API URL (Domain)', 'philia'),
            array($this, 'api_url_field_callback'),
            'philia-api-settings',
            'philia_api_settings_section'
        );

        // API Key
        add_settings_field(
            'api_key',
            __('API Key', 'philia'),
            array($this, 'api_key_field_callback'),
            'philia-api-settings',
            'philia_api_settings_section'
        );

        // Phone Meta
        add_settings_field(
            'phone_meta',
            __('Phone Meta Key', 'philia'),
            array($this, 'phone_meta_field_callback'),
            'philia-api-settings',
            'philia_api_settings_section'
        );

        // Enable Cashier Checkbox
        add_settings_field(
            'enable_cashier',
            __('Enable Cash', 'philia'),
            array($this, 'enable_cashier_field_callback'),
            'philia-api-settings',
            'philia_api_settings_section'
        );

        // Test Connection Button
        add_settings_field(
            'test_connection',
            __('Test Connection', 'philia'),
            array($this, 'test_connection_field_callback'),
            'philia-api-settings',
            'philia_api_settings_section'
        );
    }

    /**
     * Displays the Philia API settings page.
     * 
     * @return void
     */
    public static function philia_api_settings_callback() {
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            <?php
            $settings = get_option('philia_api_settings');
            if( isset( $settings['api_url'] ) && !empty( $settings['api_url'] ) && isset( $settings['api_key'] ) && !empty( $settings['api_key'] ) )
            {
                /*$phar_extension = extension_loaded('phar');
                $phar_flag = $phar_extension ? '' : '-d extension=phar.so ';
                $cron_command = "cd ". PHILIA_PLUGIN_DIR . "includes/ && ". PHP_BINDIR ."/php {$phar_flag}philia wp:sync " . ABSPATH ."wp-config.php -c";
                echo '<div class="notice notice-success">';
                    echo '<p>'. __('Please enter the following command in the cron job of your host control panel:', 'philia') .'</p>';
                    echo '<pre style="direction:ltr;text-align:left;white-space: pre-wrap; word-wrap: break-word; max-width: 100%;">'. $cron_command . '</pre>';
                echo '</div>';*/
                $res = self::create_and_send_woocommerce_api_key( $settings['api_url'] , $settings['api_key'] );
                if( wp_remote_retrieve_response_code( $res ) == 200 )
                {
                    ?>
                    <div class="notice notice-success is-dismissible">
                        <p><?php _e('The Synchronization with Philia has been successfully configured.', 'philia'); ?></p>
                    </div>
                    <?php
                }
            }
            ?>
            <form method="post" action="options.php">
                <?php
                settings_fields('philia_api_settings');
                do_settings_sections('philia-api-settings');
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    /**
     * Outputs the input field for the API URL.
     * 
     * @return void
     */
    public function api_url_field_callback() {
        $settings = get_option('philia_api_settings');
        ?>
        <input type="text" name="philia_api_settings[api_url]" value="<?php echo esc_attr($settings['api_url'] ?? ''); ?>" size="40" />
        <p class="description"><?php echo esc_attr__('Please enter the domain of the Philia web service.', 'philia'); ?></p>
        <?php
    }

    /**
     * Outputs the input field for the API key.
     * 
     * @return void
     */
    public function api_key_field_callback() {
        $settings = get_option('philia_api_settings');
        ?>
        <input type="text" name="philia_api_settings[api_key]" value="<?php echo esc_attr( $settings['api_key'] ?? ''); ?>" size="50" />
        <p class="description"><?php echo esc_attr__('Please enter the access key for the Philia web service.', 'philia'); ?></p>
        <?php
    }

    /**
     * Outputs a dropdown list of phone meta keys for synchronization.
     * 
     * @return void
     */
    public function phone_meta_field_callback() {
        global $wpdb;
        $settings = get_option('philia_api_settings');
        ?>
        <select name="philia_api_settings[phone_meta]" id="phone-meta">
            <option value=""><?php _e('Select Phone Meta', 'philia'); ?></option>
            <?php
            // Retrieve all unique phone meta keys from the usermeta table
            $phone_meta_keys = $wpdb->get_col("SELECT DISTINCT meta_key FROM {$wpdb->usermeta} WHERE meta_key LIKE '%phone%' OR meta_key LIKE '%mobile%'");

            // Populate the select box with the retrieved keys
            foreach ($phone_meta_keys as $key) {
                $selected = (isset($settings['phone_meta']) && $settings['phone_meta'] === $key) ? 'selected' : '';
                echo '<option value="' . esc_attr($key) . '" ' . $selected . '>' . esc_html($key) . '</option>';
            }
            ?>
        </select>
        <p class="description"><?php echo esc_attr__('The meta key for the phone is essential for synchronizing users with Philia. If not selected, the synchronization process will not be carried out.', 'philia'); ?></p>
        <?php
    }

    /**
     * Outputs the checkbox for enabling the cashier functionality.
     * 
     * @return void
     */
    public function enable_cashier_field_callback() {
        $settings = get_option('philia_api_settings');
        $checked = isset( $settings['enable_cashier'] ) ? (bool) $settings['enable_cashier'] : false;
        ?>
        <input type="checkbox" name="philia_api_settings[enable_cashier]" value="1" <?php checked($checked); ?> />
        <label for="philia_api_settings[enable_cashier]">
            <?php echo esc_html__('Enable', 'philia'); ?>
        </label>
        <p class="description">
            <?php echo esc_html__('By activating this option, the cash application will be enabled on the website.', 'philia'); ?>
        </p>
        <?php
    }

    /**
     * Outputs a button to test the API connection.
     * 
     * @return void
     */
    public function test_connection_field_callback() {
        ?>
        <button type="button" class="button" id="test-philia-connection"><?php _e('Test Connection', 'philia'); ?></button> <span id="test-connection-result" style="margin-top: 6px;display: inline-block;"></span>
        <p class="description"><?php _e('Test the API connection with the configured settings.', 'philia'); ?></p>
        <?php
    }

    /**
     * Enqueues necessary JavaScript files for AJAX actions on the settings page.
     * 
     * @return void
     */
    public function enqueue_scripts() {
        wp_enqueue_script('philia-api-settings-js', PHILIA_PLUGIN_URL . 'assets/philia-api-settings.js', array('jquery'), null, true);

        // Localize script with AJAX URL
        wp_localize_script('philia-api-settings-js', 'philiaApiSettings', array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'wait_text' => esc_attr__('Please wait a moment…', 'philia')
        ));
    }

    /**
     * Handles the AJAX request to test the API connection.
     * 
     * @return void Sends a JSON response indicating success or failure.
     */
    public function ajax_philia_api_test_connection() {
        $result = array();

        $response = Philia_Core_API::send_request('integration/accounts/connection/', 'GET', [], 2);

        // Check if the response is an error
        if ( is_wp_error( $response ) ) {
            // Retrieve the error code and message
            $error_code = $response->get_error_code();
            $error_message = $response->get_error_message();
            $result = array('status' => false, 'message' => sprintf( __('API Connection error - Code: %s, Message: %s', 'philia'), $error_code, $error_message) );
        } else {
            // Retrieve the response code and check for any non-success codes
            $response_code = wp_remote_retrieve_response_code($response);
            if ($response_code !== 200) { // accept only 200 as success
                $result = array('status' => false, 'message' => __('API Connection error - status: ' . $response_code, 'philia'));
            } else {
                $result = array('status' => true, 'message' => __('API Connection is established', 'philia'));
            }
        }

        wp_send_json($result);
    }

    /**
     * Handles the AJAX request to clear the API log file.
     * 
     * @return void Sends a JSON response indicating success or failure.
     */
    public function ajax_clear_philia_api_log()
    {
        $result = array();

        if ( Philia_API_Log::clear_log_file_content() ) {
            $result = array( 'status' => true, 'message' => __('Log file cleared successfully.', 'philia') );
        } else {
            $result = array( 'status' => false, 'message' => __('Failed to clear the log file content.', 'philia') );
        }

        wp_send_json( $result );
    }

    /**
     * Handles the AJAX request to clear the API log file.
     * 
     * @return void Sends a JSON response indicating success or failure.
     */
    public function ajax_clear_philia_data()
    {
        $result = array();

        $option_keys = array( 'philia_users_ids', 'philia_products_categories_ids', 'philia_products_ids', 'philia_orders_ids', 'philia_sync_users', 'philia_sync_products_categories', 'philia_sync_products', 'philia_sync_orders' );
        foreach($option_keys as $item)
        {
            update_option( $item, array() );
        }

        $result = array( 'status' => true, 'message' => __('Data cleared successfully.', 'philia') );
        wp_send_json( $result );
    }

    /**
     * Callback for the settings section description.
     * 
     * @return void
     */
    public function philia_api_settings_section_callback() {
        echo '<p>' . __('Configure the Philia API settings below.', 'philia') . '</p>';
    }

    /**
     * Sanitizes the input from the settings fields.
     * 
     * @param array $input The input from the settings form.
     * @return array The sanitized input values.
     */
    public function sanitize($input) {
        $sanitized_input = array();

        if (isset($input['enable_cashier'])) {
            $sanitized_input['enable_cashier'] = (bool)$input['enable_cashier'];
        }

        if (isset($input['api_key'])) {
            $sanitized_input['api_key'] = sanitize_text_field($input['api_key']);
        }

        if (isset($input['api_url'])) {
            $sanitized_input['api_url'] = esc_url_raw($input['api_url']);
        }

        if (isset($input['phone_meta'])) {
            $sanitized_input['phone_meta'] = sanitize_text_field($input['phone_meta']);
        }

        return $sanitized_input;
    }

    /**
     * Displays a success notice after the settings are saved.
     * 
     * @return void
     */
    public function philia_settings_saved_notice() {
        if (isset($_GET['settings-updated']) && $_GET['settings-updated'] == 'true') {
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e('Settings saved.', 'philia'); ?></p>
            </div>
            <?php
        }
    }

    /**
     * Creates and sends a WooCommerce API key to a remote server.
     *
     * @param string $philia_url The Philia service URL to register the API key.
     * @param string $philia_key The Philia service authentication key.
     *
     * @return array|WP_Error The response from the remote server or WP_Error on failure.
     */
    public static function create_and_send_woocommerce_api_key($philia_url, $philia_key)
    {
        if (!class_exists('WooCommerce')) {
            return new WP_Error('woocommerce_missing', 'WooCommerce is not active.');
        }

        global $wpdb;
        $user_id = get_current_user_id();
        $description = 'philia_api_sync';
        $permissions = 'read_write';
        $table = $wpdb->prefix . 'woocommerce_api_keys';

        // Check if an API key with the same description already exists for this user
        $existing_key = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $table WHERE user_id = %d AND description = %s LIMIT 1",
            $user_id,
            $description
        ));

        if ($existing_key)
        {
            // API key already exists, retrieve it
            $consumer_key = 'ck_' . $existing_key->truncated_key;
            $consumer_secret = $existing_key->consumer_secret;
        }
        else
        {
            // Create a new API key
            $consumer_key = 'ck_' . wc_rand_hash();
            $consumer_secret = 'cs_' . wc_rand_hash();

            $insert_result = $wpdb->insert(
                $table,
                [
                    'user_id'       => $user_id,
                    'description'   => $description,
                    'permissions'   => $permissions,
                    'consumer_key'  => wc_api_hash($consumer_key),
                    'consumer_secret' => $consumer_secret,
                    'truncated_key' => substr($consumer_key, -7),
                    'last_access'   => null,
                ],
                [
                    '%d', '%s', '%s', '%s', '%s', '%s', '%s'
                ]
            );

            if (!$insert_result) {
                return new WP_Error('api_key_creation_failed', 'Failed to create WooCommerce API key.');
            }

            // Dynamically detect WooCommerce API version
            $version = defined('WC_API_REQUEST_VERSION') ? WC_API_REQUEST_VERSION : 'v3';

            // Get the current REST API URL prefix (default is 'wp-json', but it might be changed)
            $rest_prefix = rest_get_url_prefix();

            $body = json_encode([
                'philia_url'   => $philia_url,
                'philia_key'   => $philia_key,
                'url'          => get_site_url(),
                'key'          => $consumer_key,
                'secret'       => $consumer_secret,
                'type'         => 'woocommerce',
                'version'      => $version,
                'rest_prefix'  => $rest_prefix, // Include the current REST API prefix
            ]);

            $response = wp_remote_post('http://hook.philia.vip/api/register', [
                'method'    => 'POST',
                'body'      => $body,
                'headers'   => ['Content-Type' => 'application/json'],
            ]);

            return $response;
        }
    }

    /**
     * Delete a WooCommerce API key.
     *
     * @return array|WP_Error The response from the remote server or WP_Error on failure.
     */
    public static function delete_woocommerce_api_key()
    {
        if (!class_exists('WooCommerce')) {
            return new WP_Error('woocommerce_missing', 'WooCommerce is not active.');
        }

        global $wpdb;
        $user_id = get_current_user_id();
        $description = 'philia_api_sync';
        $permissions = 'read_write';
        $table = $wpdb->prefix . 'woocommerce_api_keys';

        // Check if an API key with the same description already exists for this user
        $existing_key = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM $table WHERE user_id = %d AND description = %s LIMIT 1",
            $user_id,
            $description
        ));

        if ($existing_key)
        {
            $existing_key = $wpdb->delete( $table , array( 'user_id' => $user_id, 'description' => $description ) );
        }
    }
}

if (class_exists('Philia_API\Philia_API_Settings')) {
    Philia_API_Settings::instance();
}