<?php
/**
 * Philia Order API
 * Handles interactions with the 3rd-party API for order creation, updates, and other order actions.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/12/09 12:03:06
 * License: MIT
 */

namespace Philia_API;

class Philia_Order_API extends Philia_Core_API {

    /**
     * Static instance of the class for the singleton pattern.
     * 
     * @var Philia_Order_API|null
     */
    protected static $_instance = null;

    /**
     * Singleton instance
     * 
     * Ensures only one instance of the class is created.
     * 
     * @return Philia_Order_API The instance of this class.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Constructor.
     * Initializes the class by retrieving the API URL from WordPress settings.
     * 
     */
    public function __construct()
    {
        // Order-related hooks
        add_action('woocommerce_new_order', array(&$this, 'on_order_create'), 10, 1);
        add_action('woocommerce_update_order', array(&$this, 'on_order_update'), 10, 1);
        add_action('woocommerce_before_delete_order', array($this, 'on_order_delete')); // HPOS Compatibility
        add_action('before_delete_post', array($this, 'on_order_delete_by_post'));
    }

    /**
     * Prepare order data for API requests.
     * 
     * @param int $order_id The ID of the WooCommerce order.
     * 
     * @return array The prepared order data for the API.
     */
    public static function prepare_order_data($order_id)
    {
        $order = wc_get_order($order_id);
        if ( $order instanceof \Automattic\WooCommerce\Admin\Overrides\OrderRefund)
        {
            $order = wc_get_order( $order->get_parent_id() );
        }
        else
        {
            $order = wc_get_order($order_id);
        }
        
        $api_data = array();

        $hide_ip = 1;
        
        // Prepare items data
        $api_data['orderId'] = $order_id;
        $api_data['paymentMethod'] = $order->get_payment_method();

        $api_data['customerIp'] = $hide_ip ? '' : $order->get_customer_ip_address();
        if ( $order->get_date_created() ) {
            $api_data['dateCreated'] = $order->get_date_created()->date('Y-m-d H:i:s');
        } else {
            $api_data['dateCreated'] = $order->get_date_modified()->date('Y-m-d H:i:s');
        }
        $api_data['status'] = $order->get_status();
        $api_data['customerId'] = $order->get_customer_id();
        $api_data['customerNote'] = $order->get_customer_note();
        $api_data['itemsSubtotal'] = $order->get_subtotal();
        $api_data['shippingCost'] = (float) $order->get_shipping_total();
        $api_data['couponsTotal'] = $order->get_total_discount();
        $api_data['orderTotal'] = $order->get_total();
        $api_data['orderAfterRefund'] = $order->get_total();
        $api_data['orderCurrency'] = $order->get_currency();
        $api_data['transactionId'] = $order->get_transaction_id();
        if ( $order->get_date_paid() ) {
            $api_data['datePaid'] = $order->get_date_paid()->date('Y-m-d H:i:s');
        } else {
            $api_data['datePaid'] = $order->get_date_modified()->date('Y-m-d H:i:s');
        }

        if ( $order->get_status() === 'completed' ) {
            $date_completed = $order->get_date_completed();
            if ( $date_completed ) {
                $api_data['dateCompleted'] = $date_completed->date('Y-m-d H:i:s');
            } else {
                // Handle the case where the completed date is not set
                $api_data['dateCompleted'] = null; // or some default value
            }
        } elseif ( $order->get_status() === 'processing' ) {
            $date_paid = $order->get_date_paid();
            if ( $date_paid ) {
                $api_data['dateCompleted'] = $date_paid->date('Y-m-d H:i:s');
            } else {
                $api_data['dateCompleted'] = $order->get_date_modified()->date('Y-m-d H:i:s');
            }
        }

        // order address
        $hidden_address = 0;
        $billing_fields = array(
            'firstName' => $order->get_billing_first_name(),
            'lastName' => $order->get_billing_last_name(),
            'email' => $order->get_billing_email(),
            'phone' => $order->get_billing_phone(),
            'addressLine1' => $order->get_billing_address_1(),
            'addressLine2' => $order->get_billing_address_2(),
            'country' => $order->get_billing_country(),
            'state' => $order->get_billing_state(),
            'city' => $order->get_billing_city(),
            'postcode' => $order->get_billing_postcode(),
        );
        $api_data['billingAddress'] = $hidden_address ? [] : $billing_fields;

        $shipping_fields = array(
            'firstName' => $order->get_shipping_first_name(),
            'lastName' => $order->get_shipping_last_name(),
            'phone' => $order->get_shipping_phone(),
            'addressLine1' => $order->get_shipping_address_1(),
            'addressLine2' => $order->get_shipping_address_2(),
            'country' => $order->get_shipping_country(),
            'state' => $order->get_shipping_state(),
            'city' => $order->get_shipping_city(),
            'postcode' => $order->get_shipping_postcode()
        );
        $api_data['shippingAddress'] = $hidden_address ? [] : $shipping_fields;

        $api_data['orderNotes'] = [];
        $hide_notes = 0;
        if ( ! $hide_notes ) {

            remove_filter( 'comments_clauses', array( 'WC_Comments', 'exclude_order_comments' ) );
            foreach ( get_comments( [ 'post_id' => $order_id, 'type' => 'order_note' ] ) as $note ) {
                $orderNoteArgs = [];

                $orderNoteArgs['note']           = $note->comment_content;
                $orderNoteArgs['date']           = $note->comment_date;
                $orderNoteArgs['author']         = $note->comment_author;
                $orderNoteArgs['isCustomerNote'] = get_comment_meta( $note->comment_ID, 'is_customer_note', true ) == '1';

                $api_data['orderNotes'][] = $orderNoteArgs;
            }
        }

        $api_data['orderItems'] = [
            'line_item' => [],
            'shipping'  => [],
            'coupon'    => [],
            'fee'       => [],
            'tax'       => [],
        ];
        foreach ( $order->get_items([ 'line_item', 'shipping', 'coupon', 'fee', 'tax', ]) as $item ) {
            $mappedItemArgs = [];
            $mappedItemArgs['itemName'] = $item->get_name();
            switch ( get_class( $item ) ) {
                case 'WC_Order_Item_Product':
                    $mappedItemArgs['productId']   = $item->get_product_id();
                    $mappedItemArgs['variationId'] = $item->get_variation_id();
                    $mappedItemArgs['imageUrl']    = wp_get_attachment_image_src( get_post_thumbnail_id( $item->get_product_id() ), 'single-post-thumbnail' );
                    $mappedItemArgs['imageUrl']    = $mappedItemArgs['imageUrl'] ? $mappedItemArgs['imageUrl'][0] : NULL;
                    $mappedItemArgs['productUrl']  = get_permalink( $mappedItemArgs['productId'] );
                    $mappedItemArgs['quantity']    = $item->get_quantity();
                    $mappedItemArgs['lineTotal']   = $item->get_total();
                    $mappedItemArgs['itemMetas']   = $item->get_meta_data();

                    $api_data['orderItems']['line_item'][] = $mappedItemArgs;
                    break;
                case 'WC_Order_Item_Shipping':
                    $mappedItemArgs['methodType'] = $item->get_method_id();
                    $mappedItemArgs['cost']       = $item->get_total();

                    $api_data['orderItems']['shipping'][] = $mappedItemArgs;
                    break;
                case 'WC_Order_Item_Coupon':
                    $mappedItemArgs['itemName']       = $item->get_code();
                    $mappedItemArgs['couponId']       = $item->get_id();
                    $mappedItemArgs['discountAmount'] = $item->get_discount();

                    $api_data['orderItems']['coupon'][] = $mappedItemArgs;
                    break;
                case 'WC_Order_Item_Fee':
                    $mappedItemArgs['feeAmount'] = $item->get_amount();

                    $api_data['orderItems']['fee'][] = $mappedItemArgs;
                    break;
                case 'WC_Order_Item_Tax':
                    $mappedItemArgs['ratePercent'] = $item->get_rate_percent();
                    $mappedItemArgs['taxAmount']   = $item->get_tax_total();

                    $api_data['orderItems']['tax'][] = $mappedItemArgs;
                    break;
                default:
                    break;
            }
        }

        $api_data['refunds'] = [];
        if ( $order instanceof \Automattic\WooCommerce\Admin\Overrides\OrderRefund) {
            $api_data['refunds'][0]['orderId']        = $order->get_parent_id(); // Parent order ID of the refund
            $api_data['refunds'][0]['refundId']       = $order->get_id();
            $api_data['refunds'][0]['dateRefunded']   = $order->get_date_created()->date('Y-m-d H:i:s');
            $api_data['refunds'][0]['refundTotal']    = $order->get_total();
            $api_data['refunds'][0]['refundCurrency'] = $order->get_currency();
            $api_data['refunds'][0]['refundReason']   = $order->get_reason();

        }
        /*foreach ( $order->get_refunds() as $refund ) {
            $refundArgs = [];

            $refundArgs['dateRefunded']   = $refund->get_date_created()->date('Y-m-d H:i:s');
            $refundArgs['refundTotal']    = $refund->get_total();
            $refundArgs['refundCurrency'] = $refund->get_currency();
            $refundArgs['refundReason']   = $refund->get_reason();

            $api_data['orderAfterRefund'] += $refund->get_total();

            $api_data['refunds'][] = $refundArgs;
        }*/

        return $api_data;
    }

    /**
     * Handles order creation actions when an order is created.
     *
     * @param int $order_id The ID of the WooCommerce order being created.
     * 
     * @return void
     */
    public function on_order_create($order_id)
    {
        $api_data = self::prepare_order_data($order_id);

        // Send API request to create order
        $response = parent::send_request('wordpress/woocommerce/order/create/', 'POST', [$api_data]);

        if (is_wp_error($response)) {
            Philia_API_Log::error("Error in 'order/create/' for Order " . $order_id);
        } else {
            //Philia_API_Log::info("Order created successfully via API - Order ID: " . $order_id);
            self::add_order_id($order_id);
        }
    }

    /**
     * Handles order update actions when an order is updated.
     *
     * @param int $order_id The ID of the WooCommerce order being updated.
     * 
     * @return void
     */
    public function on_order_update($order_id)
    {
        $api_data = self::prepare_order_data($order_id);

        // Send API request to update order
        $response = parent::send_request('wordpress/woocommerce/order/update/', 'PUT', [$api_data]);

        if (is_wp_error($response)) {
            Philia_API_Log::error("Error in 'order/update/' for Order " . $order_id);
        } else {
            //Philia_API_Log::info("Order updated successfully via API - Order ID: " . $order_id);
        }
    }

    /**
     * Handles order deletion for HPOS.
     *
     * @param int $order_id The ID of the WooCommerce order being deleted.
     */
    public function on_order_delete($order_id) {
        
        // Send API request to delete order
        $response = parent::send_request('wordpress/woocommerce/order/delete/' . $order_id, 'DELETE');
        
        if (is_wp_error($response)) {
            Philia_API_Log::error("Error in 'order/delete/' for Order " . $order_id);
        } else {
            //Philia_API_Log::info("Order deleted successfully via API - Order ID: " . $order_id);
            self::remove_order_id($order_id);
        }
    }

    /**
     * Handles order deletion for post_type=shop_order (non-HPOS).
     *
     * @param int $post_id The ID of the post being deleted.
     */
    public function on_order_delete_by_post($post_id) {
        if (get_post_type($post_id) !== 'shop_order') {
            return; // Skip if the post is not an order
        }
        $this->on_order_delete($post_id);
    }

    /**
     * Adds an order ID to the 'philia_order_ids' option list.
     * 
     * @param int $order_id The order ID to be added to the list.
     * 
     * @return bool True if the order ID was added, false if it already exists.
     */
    public static function add_order_id($order_id)
    {
        if (!is_int($order_id)) {
            return false; // Invalid order ID
        }

        return parent::add_item('philia_orders_ids', $order_id);
    }

    /**
     * Removes an order ID from the 'philia_orders_ids' option list.
     * 
     * @param int $order_id The order ID to be removed from the list.
     * @return bool True if the order ID was removed, false if not found.
     */
    public static function remove_order_id($order_id)
    {
        if (!is_int($order_id)) {
            return false; // Invalid order ID
        }

        return parent::remove_item('philia_orders_ids', $order_id);
    }
}

if (class_exists('Philia_API\Philia_Order_API')) {
    Philia_Order_API::instance();
}